# Using Reinforcement Learning to Personalize Daily Step Goals for a Collaborative Dialogue with a Virtual Coach

**Author:** Martin Dierikx <br>
**Date:** September 2023

This repository is meant to show and reproduce our analyses for the paper "Using Reinforcement Learning to Personalize Daily Step Goals for a Collaborative Dialogue with a Virtual Coach".

## Contents of this Repository

```
|   ChecklistDataRepositoryReview.pdf
│   Dockerfile
│   README.md
│
└───analysis_code
│   │   Analysis questions results.ipynb
|   |   Database data preprocessing.ipynb
│   │   Demographic results.ipynb
|   |	G algorithm results.ipynb
|   |	General analysis results.ipynb
│
└───data
|   |   cleaned_database_data.csv
│   │   preprocessed_database_data.csv
│   │   preprocessed_demographic_data.csv
|   |	preprocessed_post_questionnaire_data.csv
|   |	preprocessed_pre_screening_data.csv
|   |	README.md
│
└───figures
|   |   ASA Scores.png
│   │   State_transitions.png
│
└───results
│   │   Analysis questions results.html
|   |   Database data preprocessing.html
│   │   Demographic results.html 
│   │   G algorithm results.html
|   |	General analysis.html
│
└───scripts
│   │   Pre-processing.py

```

| File/Folder name                           | Content                                                                                                        |
| ------------------------------------------ | -------------------------------------------------------------------------------------------------------------- |
| ChecklistDataRepositoryReview.pdf          | Checklist reviewing the analysis code and data, filled in by Andrei Stefan.                                    |
| Dockerfile                                 | File containing instructions to build Docker image for analyses in jupyter notebook.                           |
| README.md                                  | The file you are currently reading - provides an overview of the repositore and how to reproduce our analyses. |
| analysis_code/                             | Folder containing the jupyter notebooks with our analysis code.                                                |
| - Analysis questions results.ipynb         | Code for reproducing the results of the five analysis questions in our paper.                                  |
| - Database data preprocessing.ipynb        | Code for reproducing the preprocessing of the database data file.                                              |
| - Demographic results.ipynb                | Code for reproducing the results of the demographics table of the participants in our paper.                   |
| - G algorithm results.ipynb                | Code for reproducing the results of the G Algorithm in our paper.                                              |
| - General analysis results.ipynb           | Code for reproducing the results of the appendices and figures outside our analysis questions in our paper.    |
| data/                                      | Folder containing preprocessed data from our observational study used in our analyses.                         |
| - cleaned_database_data.csv                | Cleaned and anonimized data from the database of the conversation sessions from the study.                     |
| - preprocessed_database_data.csv           | Preprocessed data from the database of the conversation sessions from the study.                               |
| - preprocessed_demographic_data.csv        | Preprocessed data from Prolific on demographics from the participants of the study.                            |
| - preprocessed_post_questionnaire_data.csv | Preprocessed data from the post-questionnaire of the study.                                                    |
| - preprocessed_pre_screening_data.csv      | Preprocessed data from the pre-screening of the study.                                                         |
| - README.md                                | File with an overview and explanation of all the cells in the data files.                                      |
| figures/                                   | Folder containing figures created from results of our analysis.                                                |
| - ASA Scores.png                           | Figure showing the ASA Score of our chatbot and other agents from a study by Fitrianite et al.                 |
| - State_transitions.png                    | Figure showing the state transitions for a timestep of our optimal policy, corresponding to Figure 4.7.        |
| results/                                   | Folder containing html files created from the jupyter notebooks with the results after running them.           |
| - Analysis questions results.html          | Html file from running `Analysis questions results.ipynb`.                                                     |
| - Database data preprocessing.html         | Html file from running `Database data preprocessing.ipynb`.                                                    |
| - Demographic results.html                 | Html file from running `Demographic results.ipynb`.                                                            |
| - G algorithm results.html                 | Html file from running `G algorithm results.ipynb`.                                                            |
| - General analysis.html                    | Html file from running `General analysis results.ipynb`.                                                       |
| scripts/                                   | Folder containing Python scripts to preprocess the raw data files.                                             |
| - Pre-processing.py                        | Preprocessing script to clean up, anonymize and organize data.                                                 |


## Reproducing our analyses

### Initial Setup

Make sure that you have Docker installed. You can check whether you do by running `docker -v`. <br>
In case you do not have it installed, you can follow the instructions [here](https://docs.docker.com/get-docker/).

### Obtaining the Docker Image

You can build the Docker image in the directory of this README-file. <br>
Build the Docker image via `docker build . -t <DOCKER_IMAGE_NAME>`.

Replace the `<DOCKER_IMAGE_NAME>` with your chosen name of the docker image. For example, if you want to name your docker image `data_analysis`, your docker command would be `docker build . -t data_analysis`.

More information regarding `Dockerfile` and how to build images can be found [here](https://docs.docker.com/engine/reference/builder/).


### Running the Docker Container

Run the Docker container via `docker run -p 8888:8888 -e JUPYTER_ENABLE_LAB=yes -v <PATH_TO_THIS_DIRECTORY>:/home/jovyan/work <DOCKER_IMAGE_NAME>`. <br>
Replace `<PATH_TO_THIS_DIRECTORY>` with the path to the directory that this README-file is in and replace <DOCKER_IMAGE_NAME> with the name you chose for the image in the previous step.

If you are running a container where the actual path to your current folder is, for instance, `/home/users/documents`, and the name for the docker image you chose was, for example, `data_analysis`, the command should look something like this: <br>
`docker run -p 8888:8888 -e JUPYTER_ENABLE_LAB=yes -v /home/users/documents:/home/jovyan/work data_analysis`

### Running the analyses

When the container is running, go to one of the links presented in the terminal to access Jupyter Notebook.

Open the "work"-folder in Jupyter Notebook. <br>
Then, open the "analysis_code"-folder Jupyter Notebook.

Choose the .ipynb file that you want to run and reproduce the analysis.


### Results
- The `results` folder contains the .html files from running the Jupyter Notebook files. These can be obtained by running one of the .ipynb files and downloading them as .html file.


### Tables/Figures and Corresponding Files

| Figure/Table/Result from paper                       | Corresponding analysis file                              | 
|------------------------------------------------------|----------------------------------------------------------|
| Total number of participants                         | Demographic results.ipynb                                |
| Number of samples                                    | General analysis results.ipynb                           |
| Table 4.1                                            | Demographic results.ipynb                                |
| Number of people who finished the post questionnaire | General analysis results.ipynb                           |
| Table 4.2                                            | G algorithm results.ipynb                                |         
| ASA-Score                                            | General analysis results.ipynb                           |
| Figure 4.2                                           | General analysis results.ipynb, ASA Scores.png           |
| Number of rejected proposed goals                    | General analysis results.ipynb                           |
| Figure 4.3                                           | General analysis results.ipynb                           |
| Number of samples with an attained goal              | General analysis results.ipynb                           |
| Figure 4.4                                           | General analysis results.ipynb                           |
| Average steps taken before the study                 | General analysis results.ipynb                           |
| Average steps taken on the last day of the study     | General analysis results.ipynb                           |
| Figure 4.5                                           | General analysis results.ipynb                           |
| Figure 4.6                                           | General analysis results.ipynb                           |
| Figure 4.7                                           | Analysis questions results.ipynb                         |
| Figure 4.8                                           | Analysis questions results.ipynb                         |
| Figure 4.9                                           | Analysis questions results.ipynb, State_transitions.png  |
| Figure 4.10                                          | Analysis questions results.ipynb                         |
| Figure 4.11                                          | Analysis questions results.ipynb                         |
| Table 4.3                                            | Analysis questions results.ipynb                         |
| Weighted kappas                                      | Analysis questions results.ipynb                         |
| Increase in average steps per day during the study   | General analysis results.ipynb                           |
| Figure 4.12                                          | Analysis questions results.ipynb                         |
| Figure 4.13                                          | Analysis questions results.ipynb                         |
| Figure 4.14                                          | Analysis questions results.ipynb                         |
| Appendix Table D.1                                   | General analysis results.ipynb                           |
| Appendix Figure E.1                                  | General analysis results.ipynb                           |
| Appendix Figure F.1                                  | General analysis results.ipynb                           |


### Data preprocessing

The `scripts` folder contains the Python scripts used to preprocess the data files. 
Since the most of the raw data contained Prolific ids and other fields we were not allowed to publish, we could not publish those files. 
Therefore we only published the preprocessed data files. 
The Python scripts contain notes on how the raw data files looked like and what steps were taken to anonymize, clean and order the data.
For the database data, however, we published the cleaned and anonimized data file called `cleaned_database_data.csv`.
The Jupyter notebook file called `Database data preprocessing.ipynb` processes the cleaned database data and creates the `preprocessed_database_data_reproduced.csv` in the `results` folder.
Therefore, it can be used to reproduce the processed database data file and can be compared with our provided `preprocessed_database_data.csv` to check the reproducibility.

